function EMdemo
%
% Demonstrate the capabilities of EM algorithm for GMMs. Two examples are
% given: one with diagonal cov. matrices and one with full con. matrices.
%
% (c) Yannis Pantazis, 2004, csd, uoc
%     mail: pantazis@csd.uoc.gr

% FIRST EXAMPLE: DIAGONAL CASE
N = 200;
x1 = [3+2*randn(1,N), 4+1.2*randn(1,N), -1+randn(1,2*N)];
x2 = [-1+1.2*randn(1,N), 3+randn(1,N), 3+randn(1,2*N)];
x = [x1;x2];

figure(1);
plot(x1(1:N), x2(1:N), 'b.');hold on;
plot(x1(N+1:2*N), x2(N+1:2*N), 'r.');
plot(x1(2*N+1:4*N), x2(2*N+1:4*N), 'g.');hold off;
xlabel('Three classes');

% [a, m, cm, lp] = EMdiag(x, 3, 100); % without explicit initialization
[a, m, cm, lp] = EM(x, 3, 100); % without explicit initialization

fprintf(1, '\n  Results may be permutated\n  ^^^^^^^^^^^^^^^^^^^^^^^^^\n\n');
fprintf(1, 'True Values for a:        %.3f %.3f %.3f\n', [.25 .25 .5]);
fprintf(1, 'Estimated Values for a:   %.3f %.3f %.3f\n', a);
fprintf(1, '\nTrue Mean Values:        [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', [3.0 -1.0 4.0 3.0 -1.0 3.0]);
fprintf(1, 'Estimated Mean Values:   [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', m);
fprintf(1, '\nTrue Covariance matrices:\n     [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', [4.0 0.0 1.44 0.0 1.0 0.0]);
fprintf(1, '     [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', [0.0 1.44 0.0 1.0 0.0 1.0]);
fprintf(1, 'Estimated Covariance matrices:\n     [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', cm(1,:));
fprintf(1, '     [%.3f %.3f], [%.3f %.3f], [%.3f %.3f]\n', cm(2,:));
[X,Y] = meshgrid(-9.9:.1:10, -9.9:.1:10);
L = 200;
X = reshape(X, 1, L^2);
Y = reshape(Y, 1, L^2);

Z = ppe([X;Y], a, m, cm);
Z = reshape(Z, L, L);
X = reshape(X, L, L);
Y = reshape(Y, L, L);

hold on;
contour(X, Y, Z);
hold off;

figure(2);
plot(lp);
xlabel('Likelihood');

fprintf('\n Press any key to continue ...');
pause;

% SECOND EXAMPLE: FULL MATRIX CASE
N = 200;
x1 = randn(1,N);
x2 = randn(1,N);
x = [2 1;1 1]*[x1;x2];
x(1,:) = x(1,:) + 1;
x(2,:) = x(2,:) + 3;
x1 = randn(1,2*N);
x2 = randn(1,2*N);
x = [x [1.5 -1;-1 1]*[x1;x2]];
x(1,:) = x(1,:) + 0;
x(2,:) = x(2,:) + -1;

figure(1);
plot(x(1, 1:N), x(2, 1:N), 'b.');hold on;
plot(x(1, N+1:3*N), x(2, N+1:3*N), 'r.');hold off;
xlabel('Two classes');

% [a, m, cm, lp] = EMdiag(x, 3, 100); % without explicit initialization
[a, m, cm, lp] = EM(x, 2, 100); % without explicit initialization

fprintf(1, '\n\n  Results may be permutated\n  ^^^^^^^^^^^^^^^^^^^^^^^^^\n\n');
fprintf(1, 'True Values for a:        %.3f %.3f\n', [.33 .66]);
fprintf(1, 'Estimated Values for a:   %.3f %.3f\n', a);
fprintf(1, '\nTrue Mean Values:        [%.3f %.3f], [%.3f %.3f]\n', [1.0 3.0 0.0 -1.0]);
fprintf(1, 'Estimated Mean Values:   [%.3f %.3f], [%.3f %.3f]\n', m);
fprintf(1, '\nTrue Covariance matrices:\n     [%.3f %.3f], [%.3f %.3f]\n', [5.0 3.0 3.25 -2.5]);
fprintf(1, '     [%.3f %.3f], [%.3f %.3f]\n', [3.0 2.0 -2.5 2.0]);
fprintf(1, 'Estimated Covariance matrices:\n     [%.3f %.3f], [%.3f %.3f]\n', cm(1,:));
fprintf(1, '     [%.3f %.3f], [%.3f %.3f]\n', cm(2,:));
[X,Y] = meshgrid(-9.9:.1:10, -9.9:.1:10);
L = 200;
X = reshape(X, 1, L^2);
Y = reshape(Y, 1, L^2);

Z = ppe([X;Y], a, m, cm);
Z = reshape(Z, L, L);
X = reshape(X, L, L);
Y = reshape(Y, L, L);

hold on;
contour(X, Y, Z);
hold off;

figure(2);
plot(lp);
xlabel('Likelihood');
