function y = gau(x, m, s)
%
% Evaluate the density of a Gaussian probability
% 
% ----INPUT PARAMETERS----
%   x: dxL matrix wtih each column being a sample of dimension d
%   m: mean vector (dx1 matrix)
%   cm: covariance matrix (dxd matrix)
%
% ----OUTPUT PARAMETERS----
%   y: probability values (1xL matrix)
%
% (c) Yannis Pantazis, 2004, csd, uoc
%     mail: pantazis@csd.uoc.gr

d = length(s);
[wx,lx] = size(x);
y = zeros(1,lx);

ds = det(s);
si = inv(s);

% compute probability
normalize = 1/sqrt((2*pi)^d*ds);
df = x-m(:,ones(lx,1));
y = -sum(df.*(si*df)/2);
y = normalize*exp(y);

